package com.odissue.hibernate.batch;

import java.util.*;

import org.hibernate.HibernateException;
import org.hibernate.SQLQuery;
import org.hibernate.ScrollableResults;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;


public class OdissueTest {
	
	private static SessionFactory sf;
	
	public static void main (String[] args) {
		sf = new Configuration().configure().buildSessionFactory();
		Session session = sf.openSession();

		System.out.println("Get connected? - " + session.isConnected());
		
		OdissueTest ot = new OdissueTest();
		
		Integer batchTkts = ot.batchInsert("Batch Ticket", "Vladimir Frolov", 100000); // insert 100 000 rows
		ot.batchUpdate("UPDATED batch"); // update 100 000 rows
		ot.listTickets();
	}
	
	public void listTickets() {
		Session session = sf.openSession();
			List tickets = session.createQuery("FROM Ticket").list();
				for (Iterator it = tickets.iterator(); it.hasNext();) {
					Ticket tkt = (Ticket)it.next();
					System.out.println("ID: " + tkt.getId());
					System.out.println("Summary: " + tkt.getSummary());
					System.out.println("Requester: " + tkt.getRequester());
				}
	}
	
	public Integer batchInsert(String summary, String requester, int copies) {
		Session session = sf.openSession();
		Integer ticketID = null;
		Transaction tx = null;
		try {
			tx = session.beginTransaction();
			for (int i = 0; i < copies; i++) {
					Ticket tkt = new Ticket(summary, requester);
					session.save(tkt);
						if (i % 50 == 0) {  // same as the JDBC batch size
							// flush a batch of inserts and release memory:
							session.flush(); 
							session.clear();
						}
				}
			tx.commit();
		} catch (HibernateException he) {
			if (tx!=null) tx.rollback();
				he.printStackTrace();
		} finally {
			session.close();
		}
		return ticketID;
	}
	
	public void batchUpdate(String summary) {
		Session session = sf.openSession();
		Transaction tx = null;
		
		ScrollableResults cursor = session.createQuery("FROM Ticket").scroll();
		int count = 0;
		
		try {
			tx = session.beginTransaction();
			
			while (cursor.next()) {
				Ticket tkt = (Ticket)cursor.get(0);
				tkt.setSummary(summary);
				session.update(tkt);
					if (++count % 50 == 0) {
						session.flush();
						session.clear();
					}
			}
			
			tx.commit();
		} catch (HibernateException he) {
			if(tx!=null) tx.rollback();
			he.printStackTrace();
		} finally {
			session.close();
		}	
	}
	
	public void deleteTicket(Integer TicketID) {
		Session session = sf.openSession();
		Transaction tx = null;
		try {
			tx =  session.beginTransaction();
				Ticket tkt = (Ticket)session.get(Ticket.class, TicketID);
				session.delete(tkt);
				tx.commit();
		} catch (HibernateException he) {
			if(tx!=null) tx.rollback();
			he.printStackTrace();
		} finally {
			session.close();
		}
	}
}
