package com.odissue.hibernate.criteria;


import java.util.*;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.LogicalExpression;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;

import com.odissue.hibernate.criteria.WorkOrder;
import com.odissue.hibernate.criteria.Ticket;

public class OdissueTest {
	
	private static SessionFactory sf;
	
	public static void main (String[] args) {
		sf = new Configuration().configure().buildSessionFactory();
		Session session = sf.openSession();
		
		Criteria c1 = session.createCriteria(Ticket.class);
		List r1 = c1.list();
		showResults(r1, "Results #1:");
		
		Criteria c2 = session.createCriteria(Ticket.class);
		c2.add(Restrictions.eq("requester", "VF"));
		List r2 = c2.list();
		showResults(r2, "Results #2:");

		Criteria c3 = session.createCriteria(Ticket.class);
		c3.add(Restrictions.gt("id", 95));
		List r3 = c3.list();
		showResults(r3, "Results #3:");
		
		Criteria c4 = session.createCriteria(Ticket.class);
		c4.add(Restrictions.like("summary", "Ticket%"));
		c4.addOrder(Order.desc("id"));	// sorting with Order
		List r4 = c4.list();
		showResults(r4, "Results #4:");

		Criteria c5 = session.createCriteria(Ticket.class);
		c5.add(Restrictions.isNull("status"));
		List r5 = c5.list();
		showResults(r5, "Results #5:");
		
		Criteria c6 = session.createCriteria(Ticket.class);
		Criterion requester = Restrictions.eq("requester", "VF");
		Criterion id = Restrictions.gt("id", 94);
		
//		LogicalExpression orExp = Restrictions.or(requester, id);
//		c6.add(orExp);
		LogicalExpression andExp = Restrictions.and(requester, id);
		c6.add(andExp);
		
		List r6 = c6.list();
		showResults(r6, "Results #6:");
		
		// Projections and aggregations		
		Criteria c7 = session.createCriteria(Ticket.class);
//		c7.setProjection(Projections.rowCount());
//		c7.setProjection(Projections.sum("id"));
		c7.setProjection(Projections.max("id"));
		
		List r7 = c7.list();
//		System.out.println("\nResults #7: \nRows count: " + r7.get(0));
		System.out.println("\nResults #7: \nMax ID: " + r7.get(0));
		
		
	}
	
	public static void showResults(List results, String msg) {
		System.out.println("\n" + msg);
		
		for (Iterator it = results.iterator(); it.hasNext();) {
			Ticket tkt = (Ticket)it.next();
			System.out.println("ID: " + tkt.getId());
			System.out.println("Summary: " + tkt.getSummary());
			System.out.println("Requester: " + tkt.getRequester());
			
			System.out.println("Child WO: " + tkt.getWorkorder().getSummary()); 
		}
	}
	
	
}
