package com.odissue.hibernate.hql;


import java.util.*;

import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;

import com.odissue.hibernate.hql.WorkOrder;
import com.odissue.hibernate.hql.Ticket;

public class OdissueTest {
	
	private static SessionFactory sf;
	
	public static void main (String[] args) {
		sf = new Configuration().configure().buildSessionFactory();
		Session session = sf.openSession();
		
		String hql = "FROM Ticket T"; 
		Query query = session.createQuery(hql);
		List results = query.list();		
		showResults(results, "Results #1:");
		
		String hql2 = "SELECT T.id FROM Ticket T";
		Query query2 = session.createQuery(hql2);
		List results2 = query2.list();
		
		System.out.println("\nResults #2:");
		for (Iterator it = results2.iterator(); it.hasNext();) {
				System.out.println("ID: " + it.next()); 
			}
				
		String hql3 = "FROM Ticket T WHERE T.id = 95";
		Query query3 = session.createQuery(hql3);
		List results3 = query3.list();		
		showResults(results3, "Results #3:");
		
		String hql4 = "FROM Ticket T WHERE T.id < 100 ORDER BY T.id ASC";
		Query query4 = session.createQuery(hql4);
		List results4 = query4.list();		
		showResults(results4, "Results #4:");
		
		String hql5 = "FROM Ticket T WHERE T.id < 100 GROUP BY T.requester";
		Query query5 = session.createQuery(hql5);
		List results5 = query5.list();		
		showResults(results5, "Results #5:");
		
		// Named Parameters
		String hql6 = "FROM Ticket T WHERE T.id = :ticket_id";
		Query query6 = session.createQuery(hql6);
		query6.setParameter("ticket_id", 90);
		List results6 = query6.list();
		showResults(results6, "Results #6:");
		
		String hql7 = "UPDATE Ticket SET status = :t_status WHERE requester = :t_requester";
		Query query7 = session.createQuery(hql7);
		query7.setParameter("t_status", "Completed");
		query7.setParameter("t_requester", "VF");
		int results7 = query7.executeUpdate();
		System.out.println("Rows updated: " + results7);
		
		String hql8 = "DELETE FROM Ticket WHERE id = :t_id";
		Query query8 = session.createQuery(hql8);
		query8.setParameter("t_id", 90);
		int results8 = query8.executeUpdate();
		System.out.println("Rows deleted: " + results8);
		
//		String hql9 = "INSERT INTO Ticket(summary, requester, assignee) " +
//				"SELECT summary, requester, assignee FROM oldTicket";
//		Query query9 = session.createQuery(hql9);
//		int results9 = query.executeUpdate();
//		System.out.println("Rows affected: " + results9);
		
		String hql10 = "SELECT COUNT(distinct T.requester) FROM Ticket T";
		Query query10 = session.createQuery(hql10);
		List results10 = query.list();
		
		// Pagination
		String hql11 = "FROM Ticket";
		Query query11 = session.createQuery(hql11);
		query.setFirstResult(3);
		query.setMaxResults(10);
		List results11 = query11.list();
		
	}
	
	public static void showResults(List results, String msg) {
		System.out.println("\n" + msg);
		
		for (Iterator it = results.iterator(); it.hasNext();) {
			Ticket tkt = (Ticket)it.next();
			System.out.println("ID: " + tkt.getId());
			System.out.println("Summary: " + tkt.getSummary());
			System.out.println("Requester: " + tkt.getRequester());
			
			System.out.println("Child WO: " + tkt.getWorkorder().getSummary()); 
		}
	}
	
	
}
