package april26;

import java.util.Date;
import java.util.GregorianCalendar;

public class EqualsTest {
	public static void main(String[] args) {

		Employee3 peter1 = new Employee3("Peter Jackson", 180000, 1717, 12, 1);
		Employee3 peter2 = peter1;
		Employee3 peter3 = new Employee3("Peter The Great", 180000, 1717, 12, 1);
		Employee3 bob = new Employee3("Bob", 50000, 1990, 10, 1);

		System.out.println("peter1 == peter2" + (peter1 == peter2));
		System.out.println("peter1 == peter3" + (peter1 == peter3));

		System.out.println("peter1.equals(peter3)" + peter1.equals(peter3));
		System.out.println("peter1.equals(bob)" + peter1.equals(bob));

		System.out.println("bob.toString(): " + bob.toString());

		Manager3 carl1 = new Manager3("Carl Cracker", 80000, 1987, 12, 15);
		Manager3 boss = new Manager3("Carl Cracker", 80000, 1987, 12, 15);
//		boss.setBonus(50000);

		System.out.println("boss.toString(): " + boss);
		System.out.println("carl.equals(boss): " + carl1.equals(boss));
		System.out.println("peter1.hashCode(): " + peter1.hashCode());
		System.out.println("peter2.hashCode(): " + peter2.hashCode());
		System.out.println("peter3.hashCode(): " + peter3.hashCode());
		System.out.println("bob.hashCode(): " + bob.hashCode());
		System.out.println("carl1.hashCode(): " + carl1.hashCode());

	}
}

class Employee3 {

	private String name;
	private double salary;
	private Date hireDay;

	public Employee3(String n, double s, int year, int month, int day) {

		name = n;
		salary = s;
		GregorianCalendar calendar = new GregorianCalendar(year, month - 1, day);
		hireDay = calendar.getTime();
	}

	public String getName() {
		return name;
	}

	public double getSalary() {
		return salary;
	}

	public Date getHireDay() {
		return hireDay;
	}

	public void raiseSalary(double byPercent) {
		double raise = salary * byPercent / 100;
		salary += raise;
	}

	public boolean equals(Object otherObject) {

		if (this == otherObject)
			return true;

		if (otherObject == null)
			return false;

		if (getClass() != otherObject.getClass())
			return false;

		Employee3 other = (Employee3) otherObject;

		return name.equals(other.getName()) && salary == other.getSalary()
				&& hireDay.equals(other.getHireDay()); // using getters, but
														// authors use fields
	}

	public int hashCode() {

		return 7 * name.hashCode() + 11 * new Double(salary).hashCode() + 13
				* hireDay.hashCode();
	}

	public String toString() {
		return getClass().getName() + "[name = " + name + ", salary = "
				+ salary + ", hireDay = " + hireDay + "]";
	}

}

class Manager3 extends Employee3 {

	private double bonus;

	public Manager3(String n, double s, int year, int month, int day) {
		super(n, s, year, month, day);
		bonus = 0;
	}

	public double getSalary() {

		double baseSalary = super.getSalary();
		return baseSalary + bonus;
	}

	public void setBonus(double b) {
		bonus = b;
	}

	public double getBonus() {
		return bonus;
	}

	public boolean equals(Object otherObject) {

		if (!super.equals(otherObject))
			return false;

		Manager3 other = (Manager3) otherObject;
		return bonus == other.bonus;
	}

	public int hashCode() {
		return super.hashCode() + 17 * new Double(bonus).hashCode();
	}

	public String toString() {
		return super.toString() + "[bonus = " + bonus + "]";
	}

}