package odissue71;

import java.util.ArrayList;
import java.util.Iterator;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

public class TestOdissue {

	public static void main(String[] args) {
		
		ApplicationContext ctx = new ClassPathXmlApplicationContext("odissue71/odissue71.xml");
		
		// simple constructor arg bean
		Person jackTech = (Person)ctx.getBean("tech");
		jackTech.createTicket();

		// simple constructor arg + ref bean
		Person jackWriter = (Person)ctx.getBean("writer");
		jackWriter.createTicket();
		
		// factory method bean
		Dashboard dboard = (Dashboard)ctx.getBean("dboard");
		System.out.println(dboard.getCharts());
		
		// scope, init/destroy methods bean
		Incident incident = (Incident)ctx.getBean("incident");
		incident.setSummary("charts: not default");
		System.out.println(incident.getSummary());
		
		// beans are singletons (per the application context) by default, 
		// distinct instances reached with scope="prototype" 
		Incident incident2 = (Incident)ctx.getBean("incident");
		System.out.println(incident2.getSummary());
		
		
		// injecting into bean properties		
		User incRequester = (User)ctx.getBean("requester");
		System.out.println(incRequester.getUserTicket().getSummary());
		System.out.println(incRequester.getId());
		incRequester.createTicket();
		
		// injecting inner beans
		User woAssignee = (User)ctx.getBean("assignee");
		System.out.println(woAssignee.getUserTicket().getSummary());
		System.out.println(woAssignee.getUserTicket().getAssignee());
		System.out.println(woAssignee.getId());
		
		// wiring properties with p namespace
		User incRequester2 = (User)ctx.getBean("requester2");
		System.out.println(incRequester2.getUserTicket().getSummary());
		System.out.println(incRequester2.getUserTicket().getAssignee());
		System.out.println(incRequester2.getId());
		
		// wiring collections
		Person bulkUserCollect = (Person)ctx.getBean("bulkUser");
		bulkUserCollect.createTicket();
		
		// wiring maps
		BulkUser bulkUserMap = (BulkUser)ctx.getBean("bulkUserMap");
		bulkUserMap.createMapTicket();
		
		// wiring Properties collection
		BulkUser bulkUserProps = (BulkUser)ctx.getBean("bulkUserProps");
		bulkUserProps.createPropsTicket();
		
		// using Spring Expression Language (SpEL)
		User reqEL = (User)ctx.getBean("reqEL");
		System.out.println(reqEL.getId());
		System.out.println(reqEL.getUserTicket().getSummary());
		System.out.println(reqEL.getUserTicket().getAssignee());
		
		User reqEL2 = (User)ctx.getBean("reqEL2");
		System.out.println(reqEL2.getUserTicket().getSummary());
		System.out.println(reqEL2.getId());
		System.out.println(reqEL2.getUserTicket().getAssignee());
		
		User reqEL3 = (User)ctx.getBean("reqEL3");
		System.out.println(reqEL3.getUserTicket().getSummary());
		System.out.println(reqEL3.getUserTicket().getAssignee());
		
		User reqEL4 = (User)ctx.getBean("reqEL4");
		System.out.println(reqEL4.getUserTicket().getSummary());
		
		ArrayList<String> attrs = reqEL4.getAttrs();

		for(Iterator it = attrs.iterator(); it.hasNext();) {
			System.out.println(it.next());
		}
	}

}
