package odissue73.jdbc;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.jdbc.core.JdbcOperations;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

public class TestOdissue {
		
	public static void main(String[] args) throws SQLException {
				
		ApplicationContext ctx = new ClassPathXmlApplicationContext("odissue73/jdbc/odissue73jdbc.xml");
		Ticket ticket = new Ticket("This is test", "Vladimir Frolov", 21234);
		Person person = new Person("Vladimir Frolov", "RD", 10098);
		
		// persisting with JdbcTemplate
		TicketDAO tdao = (TicketDAO)ctx.getBean("ticketDAO");
//		tdao.createTicket(ticket);
		
		// persisting with JdbTemplate of custom parent class
		PersonDAO pdao = (PersonDAO)ctx.getBean("personDAO");
//		pdao.createPerson(person);

		// persisting with JdbcTemplate of Spring's 'template parent class' JdbcDaoSupport
		TicketSupportDAO tsdao = (TicketSupportDAO)ctx.getBean("ticketSupportDAO");
//		tsdao.createTicket(ticket);

		Ticket ticket2 = tdao.getTicketById(333444);
		System.out.println("FOUND: " + "\n" + ticket2.getSummary() + "\n" + ticket2.getRequester() + "\n" + ticket2.getId());
		
		Ticket ticket3 = tdao.getTicketByRequester("Al Capone");
		System.out.println(ticket3.getId() + " --- " + ticket3.getSummary());
		
		Ticket ticket4 = tdao.getTicketBySummary("Find me");
		System.out.println(ticket4.getId() + " --- " + ticket4.getRequester());
		
		System.out.println("---- ALL TICKETS ----");
		List<Ticket> tickets = new ArrayList<Ticket>();
		
		tickets = tdao.findAll();
		for (Ticket t : tickets) {
			System.out.println(t.getId() + " --- " + t.getSummary() + " --- " + t.getRequester());
		}
		
		System.out.println("---- MAPPER ALL TICKETS ----");
		List<Ticket> tickets2 = new ArrayList<Ticket>();
		tickets2 = tdao.mapperFindAll();
		for (Ticket t : tickets) {
			System.out.println(t.getId() + " --- " + t.getSummary() + " --- " + t.getRequester());
		}
		
		int total = tdao.getTotalTickets();
		System.out.println("TOTAL TICKETS: " + total);
	}

}
