package odissue73.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;

public class TicketDAO {
	
	private static JdbcTemplate template;
	private static String SQL_CREATE_TICKET = "INSERT INTO ticket VALUES (?, ?, ?)";
	private static String SQL_SELECT_TICKET = "SELECT summary, requester, id FROM ticket WHERE id = ?";
	private static String SQL_FIND_REQUESTER = "SELECT * FROM ticket WHERE requester = ?";
	private static String SQL_FIND_SUMMARY = "SELECT * FROM ticket WHERE summary = ?";
	private static String SQL_FIND_ALL = "SELECT * FROM ticket";
	private static String SQL_COUNT_TICKETS = "SELECT COUNT(*) FROM ticket";

	
	
	public JdbcTemplate getTemplate() {
		return template;
	}

	public static void setTemplate(JdbcTemplate template) {
		TicketDAO.template = template;
	}

	public static void createTicket(Ticket tkt) {		
		template.update(SQL_CREATE_TICKET, tkt.getSummary(), tkt.getRequester(), tkt.getId());
		System.out.println("TICKET CREATED");
	}
	
	public Ticket getTicketById(int id) {
		return template.queryForObject(SQL_SELECT_TICKET,
				new ParameterizedRowMapper<Ticket>() {
			public Ticket mapRow(ResultSet rs, int rowNum) throws SQLException {
				Ticket ticket = new Ticket();
				ticket.setSummary(rs.getString(1));
				ticket.setRequester(rs.getString(2));
				ticket.setId(rs.getInt(3));
				return ticket;
			}
		}, id);
	}
	
	// custom RowMapper
	public Ticket getTicketByRequester(String r) {
		Ticket ticket = template.queryForObject(SQL_FIND_REQUESTER, new Object[] { r }, new TicketRowMapper());
		return ticket;
	}
	
	// BeanPropertyRowMapper
	public Ticket getTicketBySummary(String s) {
		Ticket ticket = template.queryForObject(SQL_FIND_SUMMARY, 
				new Object[] { s }, new BeanPropertyRowMapper(Ticket.class));
		return ticket;
	}
	
	// Multiple rows manually mapped
	public List<Ticket> findAll() {
		
		List<Ticket> tickets = new ArrayList<Ticket>();
		
		List<Map<String, Object>> rows = template.queryForList(SQL_FIND_ALL);
		for (Map row : rows) {
			Ticket ticket = new Ticket();
			ticket.setSummary((String)row.get("SUMMARY"));
			ticket.setRequester((String)row.get("REQUESTER"));
			ticket.setId((Integer)row.get("ID"));
			
			tickets.add(ticket);
		}
		return tickets;
	}
	
	// Multiple rows mapped with BeanPropertyRowMapper
	public List<Ticket> mapperFindAll() {
		
		List<Ticket> tickets = template.query(SQL_FIND_ALL, 
				new BeanPropertyRowMapper(Ticket.class));
		
		return tickets;
	}
	
	public int getTotalTickets() {
		int total = template.queryForInt(SQL_COUNT_TICKETS);
		return total;
	}

}
