package com.odissue.mvc;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

@Controller
@RequestMapping("/issues")
public class IssueController {

	List<Ticket> tickets = new ArrayList<Ticket>();
	TicketDAO tdao = null;
	int total;
	int marker = 0;
	
	public void initTickets() {
		ApplicationContext ctx = new ClassPathXmlApplicationContext("com/odissue/mvc/odissue-jdbc.xml");
		TicketDAO tdaoTemp = (TicketDAO)ctx.getBean("ticketDAO");
 
		tickets = tdaoTemp.mapperFindAll();
		total = tdaoTemp.getTotalTickets();
		
		tdao = tdaoTemp;
		marker++;
	}

	public List<Ticket> getTickets() {
		if(marker < 1) {
			initTickets();
		}
		return tickets;
	}
	
	
	// --- LIST --- //
	@RequestMapping("/")
	public String listTickets(Model model) {
		model.addAttribute("tickets", getTickets());
		model.addAttribute("total", total);
		return "tickets/listTickets";		
	}

	
	// --- NEW --- //
	@RequestMapping(method=RequestMethod.GET, params="new")
	public String newTickets(Model model) {
		model.addAttribute("ticket", new Ticket());
		return "tickets/createForm";
	}
	
	@RequestMapping(method=RequestMethod.POST)
	public String addTickets(Ticket ticket, BindingResult bindingResult) {
		if(bindingResult.hasErrors()) {
			return "tickets/createForm";	}	
		tdao.createTicket(ticket);
		
		initTickets();
		return "redirect:/issues/ticket?id=" + ticket.getId();		
	}
	
	
	// --- FIND --- //
	@RequestMapping(value="/search", method=RequestMethod.GET, params="find")
	public String searchTickets(Model model) {
		model.addAttribute("ticket", new Ticket());
		return "tickets/findForm";
	}
	
	@RequestMapping(value="/ticket", method=RequestMethod.GET)
	public String findTicketsById(@RequestParam("id")int id, Model model) {
		initTickets();
		
		List<Ticket> foundTickets = new ArrayList<Ticket>();
		
		for(Iterator it = tickets.iterator(); it.hasNext(); ) { 
			Ticket tkt = (Ticket)it.next();
				if(tkt.getId() == id) {
					foundTickets.add(tkt);	
				} 
		}
		
		model.addAttribute("foundTickets", foundTickets);
		model.addAttribute("id", id);
		return "tickets/viewFindById";
	}
	
	@RequestMapping(value="/search", method=RequestMethod.POST)
	public String findTickets(Ticket ticket, BindingResult bindingResult) {
		if(bindingResult.hasErrors()) {
			return "issues/new";	}	
		
		
		return "redirect:/issues/ticket?id=" + ticket.getId();		
	}

	
	// --- CREATE WO --- //
	@RequestMapping(value="/ticket", method=RequestMethod.POST)
	public String createWorkOrder(@RequestParam("id")int id, Model model) {
			
		Ticket ticket = new Ticket();
		
		for(Iterator it = tickets.iterator(); it.hasNext(); ) { 
			Ticket tkt = (Ticket)it.next();
				if(tkt.getId() == id) {
					ticket = tkt;
				} 
		}
		
		tdao.createWorkOrder(ticket);
		initTickets();
		
		return "redirect:/issues/";
	}
	
	
	// --- ACTIONS --- //
	@RequestMapping(value="/actions", method=RequestMethod.GET)
	public String getActions(Model model) {
		model.addAttribute("tickets", getTickets());
		return "actions/listActions";
	}
	
	// --- ACTIONS: CLOSE ALL --- //
	@RequestMapping(value="/actions/close", method=RequestMethod.GET)
	public String viewCloseAll(Model model) {
		model.addAttribute("tickets", getTickets());
		return "/actions/closeForm";
	}
	
	@RequestMapping(value="/actions/close", method=RequestMethod.POST)
	public String actionCloseAll(Model model) {
		tdao.closeAllTickets();
		
		initTickets();		
		model.addAttribute("tickets", getTickets());
		return "redirect:/issues/";
	}	
	
	// --- ACTIONS: CHANGE ASSIGNEE --- //	
	@RequestMapping(value="/actions/changeAssignee", method=RequestMethod.GET)
	public String viewChangeAssigneeForm(Model model) {
		model.addAttribute("ticket", new Ticket());
		model.addAttribute("tickets", getTickets());
		return "/actions/changeAssigneeForm";
	}
	
	@RequestMapping(value="/actions/changeAssignee", method=RequestMethod.POST)
	public String processChangeAssigneeForm(Ticket ticket, BindingResult bindingResult, Model model) {
		tdao.changeAssignee(ticket);
		
		initTickets();
		model.addAttribute("id", ticket.getId());
		model.addAttribute("assignee", ticket.getAssignee());		
		
		return "/actions/changeAssigneeAction";		
	}

}
